module Tiramisu
  class Expectation

    def self.restore_object_status obj
      return unless obj.instance_variable_get(:@__tiramisu__original_methods__)

      obj.instance_variable_get(:@__tiramisu__original_methods__).each_pair do |n,m|
        obj.define_singleton_method(n, &m)
      end

      obj.instance_variable_set(:@__tiramisu__original_methods__, {})
      obj.instance_variable_set(:@__tiramisu__received_messages__, {})
    end

    attr_reader :expected_message, :received_messages

    def initialize object, expected_message, assert, caller
      @object = object
      @expected_message = expected_message.to_sym
      @assert = assert
      @caller = caller
      proxify(@object, @expected_message)
    end

    def validate
      @received_messages = @object.__tiramisu__received_messages__[expected_message] || []
      return refute_message_received unless @assert
      assert_message_received
      assert_message_received_with_correct_arguments
      assert_message_returned_correct_value
      assert_message_raised_as_expected
      assert_message_thrown_as_expected
    end

    private
    def proxify object, method
      mount_original_methods_depot(object)
      mount_received_messages_depot(object)
      store_original_method(object, method)

      object.define_singleton_method method do |*a,&b|
        log = {arguments: a, block: b, caller: Tiramisu.relative_location(caller[0])}
        (__tiramisu__received_messages__[method] ||= []).push(log)

        if __tiramisu__original_methods__[method]
          return begin
            log[:returned] = __tiramisu__original_methods__[method].call(*a, &b)
          rescue UncaughtThrowError => e
            log[:thrown] = Tiramisu.extract_thrown_symbol(e)
          rescue Exception => e
            log[:raised] = e
          end
        end

        if respond_to?(:method_missing)
          return begin
            log[:returned] = __send__(:method_missing, method, *a, &b)
          rescue UncaughtThrowError => e
            log[:thrown] = Tiramisu.extract_thrown_symbol(e)
          rescue Exception => e
            log[:raised] = e
          end
        end

        log[:raised] = NoMethodError.new("undefined method `%s' for %s:%s" % [method, self.inspect, self.class])
      end
    end

    def mount_original_methods_depot object
      return if object.respond_to?(:__tiramisu__original_methods__)
      def object.__tiramisu__original_methods__; @__tiramisu__original_methods__ ||= {} end
    end

    def mount_received_messages_depot object
      return if object.respond_to?(:__tiramisu__received_messages__)
      def object.__tiramisu__received_messages__; @__tiramisu__received_messages__ ||= {} end
    end

    def store_original_method object, method
      return unless object.respond_to?(method)
      object.__tiramisu__original_methods__[method] ||= object.method(method)
    end

    def assert_message_received
      Tiramisu.fail('Expected %s to receive %s message' % [
        Tiramisu.pp(@object),
        Tiramisu.pp(expected_message)
      ], @caller) if received_messages.empty?
    end

    def refute_message_received
      Tiramisu.fail('Not Expected %s to receive %s message' % [
        Tiramisu.pp(@object),
        Tiramisu.pp(expected_message)
      ], @caller) if received_messages.any?
    end
  end
end

require 'tiramisu/expectations/with'
require 'tiramisu/expectations/return'
require 'tiramisu/expectations/raise'
require 'tiramisu/expectations/throw'
