module Tiramisu
  class Expectation

    # ensure received message raises as expected
    #
    # @note if block given it will have precedence over arguments
    #
    # @example
    #   x = mock(X.new)
    #   expect(x).to_receive(:y).and_raise(NoMethodError)
    #   # call `x.y` for test to pass
    #
    def and_raise type = nil, message = nil, &block
      @raise = block || [type, message]
    end

    def assert_message_raised_as_expected
      return unless @raise
      if @raise.is_a?(Proc)
        received_messages.find {|log| @raise.call(log[:raised])} || Tiramisu.fail([
          'Looks like :%s message never raised as expected' % expected_message,
          'See validation block'
        ], @caller)
      else
        received_messages.each do |log|
          next unless f = Tiramisu.assert_raised_as_expected(log, *@raise)
          Tiramisu.fail(f, log[:caller])
        end
      end
    end
  end
end
