module Tiramisu
  class Expectation

    # ensure received message returns expected value
    #
    # @note if block given it will have precedence over arguments
    #
    # @example
    #   n = mock(1)
    #   expect(n).to_receive(:+).with(1).and_return(2)
    #
    def and_return value = nil, &block
      @return = block || value
    end

    private
    def assert_message_returned_correct_value
      return unless @return
      if @return.is_a?(Proc)
        received_messages.find {|log| @return.call(log[:returned])} || Tiramisu.fail([
          'Looks like :%s message never returned expected value' % expected_message,
          'See validation block'
        ], @caller)
      else
        received_messages.find {|log| log[:returned] == @return} || Tiramisu.fail([
          'Looks like :%s message never returned expected value:' % expected_message,
          Array(@return).map {|x| Tiramisu.pp(x)}.join(', ')
        ], @caller)
      end
    end
  end
end
