module Tiramisu
  class Expectation

    # ensure expected message received with correct arguments
    #
    # @note if block given it will have precedence over arguments
    #
    # @example
    #   test :some_test do
    #     some_object = mock(SomeObject.new)
    #     expect(some_object).to_receive(:some_method).with(:some, :args)
    #     # call `some_object.some_method(:some, :args)` for test to pass
    #   end
    #
    def with *args, &block
      @with = block || args
      self
    end

    private
    def assert_message_received_with_correct_arguments
      return unless @with
      if @with.is_a?(Proc)
        received_messages.find {|log| @with.call(log[:arguments])} || Tiramisu.fail([
          'Looks like :%s message never was called with expected arguments' % expected_message,
          'See validation block'
        ], @caller)
      else
        received_messages.find {|log| log[:arguments] == @with} || Tiramisu.fail([
          'Looks like :%s message never was called with expected arguments:' % expected_message,
          Array(@with).map {|x| Tiramisu.pp(x)}.join(', ')
        ], @caller)
      end
    end
  end
end
