module Tiramisu

  def assert_raised_as_expected object, expected_type = nil, expected_message = nil, block = nil
    f = assert_raised(object)
    return f if f

    return assert_raised_as_expected_by_block(object, block) if block

    if expected_type
      f = assert_raised_expected_type(object, expected_type)
      return f if f
    end

    if expected_message
      f = assert_raised_expected_message(object, expected_message)
      return f if f
    end
    nil
  end

  def assert_raised object
    return [
      'Expected a exception to be raised at %s' % object[:caller]
    ] unless object[:raised]
    nil
  end

  def assert_raised_as_expected_by_block object, block
    return [
      'Looks like wrong or no error raised at %s' % object[:caller],
      'See validation block'
    ] unless block.call(object[:raised])
    nil
  end

  def assert_raised_expected_type object, expected_type
    return [
      'Expected a %s to be raised at %s' % [expected_type, object[:caller]],
      'Instead a %s raised' % object[:raised].class
    ] unless object[:raised].class == expected_type
    nil
  end

  def assert_raised_expected_message object, expected_message
    regexp = expected_message.is_a?(Regexp) ? expected_message : /\A#{expected_message}\z/
    return [
      'Expected the exception raised at %s' % object[:caller],
      'to match "%s"' % regexp.source,
      'Instead it looks like',
      pp(object[:raised].message)
    ] unless object[:raised].message =~ regexp
    nil
  end
end
