module Tiramisu

  def refute_raised_as_expected object, expected_type, expected_message, block = nil
    f = refute_raised(object, expected_type || expected_message)
    return f if f

    if expected_type
      f = refute_raised_expected_type(object, expected_type)
      return f if f
    end

    if expected_message
      f = refute_raised_expected_message(object, expected_message)
      return f if f
    end
    nil
  end

  def refute_raised object, should_raise = false
    if should_raise
      return [
        'Expected a exception to be raised at %s' % object[:caller]
      ] unless object[:raised]
    else
      return [
        'A unexpected exception raised at %s' % object[:caller],
        object[:raised]
      ] if object[:raised]
    end
    nil
  end

  def refute_raised_expected_type object, expected_type
    return [
      'Not expected a %s to be raised at %s' % [object[:raised].class, object[:caller]],
    ] if object[:raised].class == expected_type
    nil
  end

  def refute_raised_expected_message object, expected_message
    regexp = expected_message.is_a?(Regexp) ? expected_message : /\A#{expected_message}\z/
    return [
      'Not expected raised exception to match %s' % regexp.source,
      object[:raised]
    ] if object[:raised].message =~ regexp
    nil
  end
end
